require File.expand_path(File.join(File.dirname(__FILE__), '..', '..', '..', 'puppet_x', 'redhat', 'ifcfg.rb'))

Puppet::Type.type(:vs_port).provide(:ovs_redhat, :parent => :ovs) do
  desc 'Openvswitch port manipulation for RedHat OSes family'

  confine    :osfamily => :redhat
  defaultfor :osfamily => :redhat

  commands :ip     => 'ip'
  commands :ifdown => 'ifdown'
  commands :ifup   => 'ifup'
  commands :vsctl  => 'ovs-vsctl'

  def create
    unless vsctl('list-ports', @resource[:bridge]).include? @resource[:interface]
      super
    end

    if interface_physical?
      IFCFG::Port.new(@resource[:interface], @resource[:bridge]).save

      if link?
        if dynamic?
          # Persistent MAC address taken from interface
          bridge_mac_address = File.read("/sys/class/net/#{@resource[:interface]}/address")
          IFCFG::BridgeDynamic.new(@resource[:bridge], @resource[:interface], bridge_mac_address).save
        else
          device = ip('addr', 'show', @resource[:interface])
          cidr = device.to_s.match(/inet (\d*\.\d*\.\d*\.\d*\/\d*)/)
          cidr = cidr ? cidr[1] : ""
          IFCFG::BridgeStatic.new(@resource[:bridge], cidr).save
        end
        ifdown(@resource[:interface])
        ifdown(@resource[:bridge])
        ifup(@resource[:interface])
        ifup(@resource[:bridge])
      else
        IFCFG::Bridge.new(@resource[:bridge]).save
      end
    end
  end

  def exists?
    if interface_physical?
      super &&
      IFCFG::OVS.exists?(@resource[:interface]) &&
      IFCFG::OVS.exists?(@resource[:bridge])
    else
      super
    end
  end

  def destroy
    if interface_physical?
      ifdown(@resource[:bridge])
      ifdown(@resource[:interface])
      IFCFG::OVS.remove(@resource[:interface])
      IFCFG::OVS.remove(@resource[:bridge])
    end
    super
  end

  private

  def dynamic?
    device = ''
    device = ip('addr', 'show', @resource[:interface])
    return device =~ /dynamic/ ? true : false
  end

  def link?
    # Cannot use commands for grep!
    File.read("/sys/class/net/#{@resource[:interface]}/operstate") =~ /up/
  end

  def interface_physical?
    # OVS port don't have entries in /sys/class/net

    # Alias interfaces (ethX:Y) must use ethX entries
    interface = @resource[:interface].sub(/:\d/, '')
    ! Dir["/sys/class/net/#{interface}"].empty?
  end
end
